#include "main.h"
#include "uart.h"

/*=====================================================================================*/
/***************************************************************************************/
/*
 * Main Sector                                                                         
 */
/***************************************************************************************/
/*=====================================================================================*/


//*******************************************************************
// Initialize timer0 to use the main crystal clock and the output
// compare interrupt feature to generate an interrupt approximately
// once per millisecond to use as a general purpose time base.
//*******************************************************************

void
init_timer0(void)
{
    // ====================================================
    // Timer 0 (laueft im Milisekunden Takt
    // ====================================================

    TCCR0 = 0;

    // Timer/Counter0 Output Compare Flag (OCF0) loeschen
    //                Overflow Flag (TOV0) loeschen

    TIFR |= _BV(OCF0) | _BV(TOV0);

    // Output Compare Interrupt enablen

    TIMSK |= _BV(TOIE0) | _BV(OCIE0);

    // - Prescale auf 1024
    // - OCR0 auf 125 bei 16 Mhz oder 115 bei 14.7456 Mhz

    TCCR0 = _BV(WGM01) | _BV(CS02) | _BV(CS00);
    TCNT0 = 0;
    OCR0 = OCR_1MS;

    // ====================================================
    // Timer 1 (laueft im Sekunden Takt
    // ====================================================

    TCCR1B = 0;

    // Prescale auf 1024, OCR1A auf 14400 ergibt 1 Sekunde 

    TCCR1B = _BV(WGM12) | _BV(CS12) | _BV(CS10);

    OCR1A = 14400;

    TIMSK |= _BV(TOIE1) | _BV(OCIE1A);

}

//*******************************************************************
// Timer0 Overflow Interrupt Service Routine
//*******************************************************************

ISR(TIMER0_COMP_vect)
{
    ms_count++;
}

//*******************************************************************
// Sleep Routine
//*******************************************************************

void
ms_sleep(uint16_t ms)
{
    TCNT0 = 0;
    ms_count = 0;
    while (ms_count != ms);
}

//*******************************************************************
// Reset System Counter
//*******************************************************************

void
reset_sys_timer(void)
{
    cli();
    sys_timer = 0;
    sei();
}

//*******************************************************************
// Timer1 Overflow Interrupt Service Routine
// Dieser Timer laeuft im Sekundentakt. 
//*******************************************************************

ISR(TIMER1_COMPA_vect)
{
    // Jede Sekunde die Zeit von der RTC holen

    // ds1307_read_date_time();
    sec_timer++;

    // LED auf Board blinken lassen, damit man sieht, dass die Kiste noch laeuft.

    led_toggle();
}

//****************************************************************************************
// Main Sector                                                                        
//****************************************************************************************

int
main(void)
{
    char buffer[20];
    char h_buf[5];
    uint8_t x =0;
    uint8_t	h_count = 0;

    char uart_buffer[128];

    uint32_t speed_g = 0;
    uint32_t speed_n = 0;

	uint32_t km_1 = 0;

    // Init Stuff

    init_timer0();

    // Init UART0 with (maybe) PC behind

    uart_init(UART_BAUD_SELECT(UART0_BAUD, F_CPU));

    // Init UART1 with GPS behind

    sei();

    ms_sleep(1000);

    uart_puts_P("\r\nTest Progi\r\n");

    sprintf(buffer,"1.39");

    while (1) {
		
    	sprintf(uart_buffer,"\r\nBuffer: %s\r\n",buffer);
		uart_puts(uart_buffer);
		
    	x =0;
    	h_count = 0;
		km_1 = 0;

        // Zuerst mal alles vor dem Punkt (.) in den Hilfsbuffer h_buf schreiben, dabei die
        // Anzahl der Stellen in h_count zaehlen

        h_count = 0;
        while (buffer[x] != '.')
        {
                h_buf[h_count++] = buffer[x++];
        }

        // Nun auf Grund der gefundenen Stellen den Ganzahlteil der Variablen Speed fuellen
        // Achtung, h_count beginnt bei 0 also bedeutet 2, dass 3 Stellen vor dem Komma waren ;-)

        if (h_count == 3)
        {
                speed_g = (h_buf[0]-48)*100 + (h_buf[1]-48)*10 + (h_buf[2]-48);
        } else if (h_count == 2)
        {
                speed_g = (h_buf[0]-48)*10 + (h_buf[1]-48);
        } else
        {
                speed_g = (h_buf[0]-48);
        }

        // Jetzt kommt der Nachkommateil, der scheint immer zweistellig zu sein
	
        x++; // zuerst eines weiter, wir stehen sonst auf dem Dezimalpunkt

        speed_n = (buffer[x++]-48)*10;
        speed_n += (buffer[x++]-48);

		// Nun haben wir den Nachkommateil in "speed_n"

        sprintf(uart_buffer,"Step1: g: %lu n: %lu km_1: %lu \r\n",speed_g, speed_n, km_1);
		uart_puts(uart_buffer);

        // Nun die Knoten in Km/h umrechnen
		// Bei 1.39 Knoten sollte speed_g = 1 
		//                        speed_n = 39 sein
		//
		// 1 * 1852        = 1852 m/h
		// (39 * 1852)/100 = 722.28 m/h (.28 faellt weg)
		//                 = 2574 m/h
		// ergibt speed_g  = 2
		//        speed_n  = 574

        km_1 = (speed_g * 1852) + ((speed_n * 1852) / 100); 

        sprintf(uart_buffer,"Step2: g: %lu n: %lu km_1: %lu \r\n",speed_g, speed_n, km_1);
		uart_puts(uart_buffer);

        speed_g = km_1 / 1000;
        speed_n = km_1 % 1000;

        sprintf(uart_buffer,"Step2: g: %lu n: %lu km_1: %lu \r\n",speed_g, speed_n, km_1);
		uart_puts(uart_buffer);

		ms_sleep(3000);
    } 
}
